@@ -1,26 +1,29 @@
-/* DBMS Server
- * $Id: children.c,v 1.5 1998/12/19 19:43:29 dirkx Exp $
+/*
+ *     Copyright (c) 2000-2004 Alberto Reggiori <areggiori@webweaving.org>
+ *                        Dirk-Willem van Gulik <dirkx@webweaving.org>
  *
- * (c) 1998 Joint Research Center Ispra, Italy
- *     ISIS / STA
- *     Dirk.vanGulik@jrc.it
+ * NOTICE
  *
- * based on UKDCils
+ * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
+ * file you should have received together with this source code. If you did not get a
+ * a copy of such a license agreement you can pick up one at:
  *
- * (c) 1995 Web-Weaving m/v Enschede, The Netherlands
- *     dirkx@webweaving.org
+ *     http://rdfstore.sourceforge.net/LICENSE
+ *
+ * DBMS Server
  *
  * Dealing with birth(control) of children, handing of
  * tasks and subsequent reaping of accidentally terminal
  * cases. 
  *
- * Strategy/Design note
+ * Strategy/Design/Bumpf
+ *
  * 	The idea here is that each time a new/init comes
  *	in from a client; we check if we already have the database
  *	requested open (note; we assume a strict one socket
  *	per database relation). If one of our children already
  *	has the database openened; hand off the connection to
- * 	that child and _FORGET_ about it.
+ * 	that child and forget about the connection.
  *
  *	This way we do not have to worry about locks; as each
  * 	dbm has just one process handling it.
@@ -32,92 +35,134 @@
  *	able to handle RQ's more independently; thus allowing
  *	better utilizition of the bus to the disk; as we have
  *	decoupled the H function.
+ *
+ *	For now we changed to a fork(); rather than pull from a 
+ *	thread pool as in the UKDCils. The main reasons are that
+ *	we are worried about one DB blowing up another, and that
+ *	some of the older FreeBSD production boxes give the same
+ *	problems we had with the IMS user land hangs. But really
+ *	treading would be better; and just require a few mutexi-es.
+ *
+ * $Id: children.c,v 1.19 2004/08/19 18:57:36 areggiori Exp $
  */                                   
 #ifdef FORKING
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <unistd.h>
-
-#include <fcntl.h>
-#include <time.h>
-#include <string.h>
-#include <signal.h>
-
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/errno.h>
-#include <sys/uio.h>
-
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <db.h>
 
 #include "dbms.h"
+#include "dbms_comms.h"
 #include "dbmsd.h"
 
 #include "deamon.h"
 #include "handler.h"
+#include "children.h"
 #include "mymalloc.h"
-                                                             
-#define CMSG_LEN(x) (ALIGN(sizeof(struct cmsghdr)) + ALIGN(x))
-#define CMSG_SPACE(x) ( ALIGN(sizeof(struct cmsghdr)) + x)
 
+// unpatched solaris and FreeBSD<2.2 need them. I think.
+//
+#ifndef ALIGN
+#define ALIGN(x) (x)
+#endif
+
+#ifndef CMSG_LEN      
+#if RDFSTORE_PLATFORM_SOLARIS
+#define CMSG_LEN(x) (ALIGN(sizeof(struct cmsghdr)) + ALIGN(x))
+#else
+#define CMSG_LEN(x) (_CMSG_HDR_ALIGN(x))
+#endif
+#endif
+
+#ifndef CMSG_SPACE
+#if RDFSTORE_PLATFORM_SOLARIS
+#define CMSG_SPACE(x) ( ALIGN(sizeof(struct cmsghdr)) + x)
+#else
+#define CMSG_SPACE(x) (_CMSG_DATA_ALIGN(x))
+#endif
+#endif
+
+#ifdef STATIC_BUFF
+static child_rec * free_child_list = NULL;
+static int free_child_list_len = 0;
+static int free_child_list_keep= 2;
+static int free_child_list_max = 4;
+#endif
+int child_counter = 0;
+
+// XXX inline or macro :-) most of the following functions are
+// just used in one or two places..
 
 int atomic_send(
 	int fd,
-  	struct msghdr * msg
+  	struct msghdr * msg,
+	int tosend
 	)
 {
-  int tosend,i;
+  int n;
+  assert(tosend != 0); // otherwise we cannot detect a child close
 
-  for(tosend=0,i=0;i<msg->msg_iovlen;i++)
-	tosend += msg->msg_iov[i].iov_len;
+retry_snd:
+  /* XXX does MSG_WAITALL actually work ?? 
+   * SCO acts strange compared to BSDi....
+   * we normally would do a proper loop;
+   * but this should be atomic accourding
+   * to the BSD man page. 
+   */
+   n=sendmsg(fd,msg,0); 
 
-  while(1) { 
-	/* XXX does MSG_WAITALL actually work ?? */
-	int n=sendmsg(fd,msg,0); 
+   if ((n<0) && (errno=EAGAIN))
+	goto retry_snd;
 
-        if ((n<0) && ((errno=EAGAIN) || (errno=EINTR))) 
-		continue; 
-	if (n<0) { 
-		log(L_ERROR,"Could not atomically send msg: %s",strerror(errno)); 
-		return -1; 
-		}; 
-	if ((n==0) && (tosend)) { 
-		log(L_ERROR,"Child closed connection"); 
-		return -1; 
-		}; 
-	if (n==tosend)
-		return 0;
-	log(L_FATAL,"Assumption wrong, expected atomic %d = %d",n,tosend);
-	return -1;
-	}
+   if ((n<=0) && (errno=EINTR)) 
+	goto retry_snd;
+ 
+   if (n<0) { 
+	dbms_log(L_ERROR,"Could not atomically send msg: %s",strerror(errno)); 
+	return -1; 
+	} 	
+   else
+   if (n==0) {
+	dbms_log(L_ERROR,"Child closed connection"); 
+	return -1; 
+	} 
+
+   assert(n==tosend); // who trust the man page ?
+   return 0;
 }
 
 void free_child( child_rec * r) 
 {
-	if (r->r) 
-		r->r->close = 1;
+	dbms_log(L_DEBUG,"Freeing child %x %d",r,r->pid);
+	if (r->r) {
+		dbms_log(L_DEBUG,"And marking close fd=%d",r->r->clientfd);
+		r->r->close = 1; MX;
+		if (mum_pid)
+			zap(r->r);/* XXXXXXXX */
+	};
+
+#ifdef STATIC_BUFF
+	if (free_child_list_len < free_child_list_keep) {
+		r->nxt = free_child_list;
+		assert( r != free_child_list);
+		free_child_list = r;
+		free_child_list_len ++;
+	} else
+#endif
 	myfree(r);
+
+	child_counter--;
 }
 	
 void zap_child( child_rec * r)
 {
 	child_rec * * p;
 
-	log(L_DEBUG,"Zapped memory for a child");
+	dbms_log(L_DEBUG,"Zapped memory for a child");
         for ( p = &children; *p && *p != r; )
                 p = &((*p)->nxt);
 
-	if (*p == NULL)
-		log(L_ERROR,"Zapping unkown child ? children=%p",children);
+	//if (*p == NULL)
+	//	dbms_log(L_ERROR,"Zapping unkown child ? children=%p",children);
+	assert( *p );
 
 	*p = r->nxt;
-
 	free_child(r);
 }
 
@@ -135,6 +180,14 @@ clean_children( void )
 	children=NULL;
 }
 
+/* Create a new child/thread. And then hand over the file descriptor of the current
+ * incoming connection.
+ *
+ * Return values:
+ *	ptr			child created; ptr to record with details.
+ *	null, errno = 0		we are the child.
+ *	noll, errno != 0	error occured.
+ */
 child_rec *
 create_new_child( void )
 {
@@ -144,29 +197,38 @@ create_new_child( void )
 	 */
 	int pipefd[2];
 	child_rec * child = NULL;
-	pid_t pid;
-	log(L_INFORM,"Creating new child");
+	pid_t pid,this_pid;
+	dbms_log(L_INFORM,"Creating new child");
 
 	if ((socketpair(AF_UNIX, SOCK_STREAM,0,pipefd))<0)	
 		return NULL;
 
-	if ( (child = (struct child_rec *) 
-		mymalloc(sizeof(struct child_rec))) == NULL )
-			return NULL;
+/*
+	XXXX
+
+	Braindead... we fork and then always clean all the
+	child cruft; as that child does not need to have
+	global knowledge. i.e. we faul the page and cause
+	a copy on first write and all that. Multi treading
+	used to solve this; but no longer with the new fork().
+*/
+
+	this_pid=getpid();
 	pid=fork();
 	if (pid <0 ) {
-		log(L_ERROR,"Failed to create a child: %s",strerror(errno));
-		myfree(child);
+		dbms_log(L_ERROR,"Failed to create a child: %s",strerror(errno));
+
 		return NULL;
 		} else
 	if (pid == 0) {
                 connection * r;
 		struct sigaction 	act,oact;
+		int mum_fd = pipefd[1];
+		mum_pid = this_pid;
 
-		mum_fd = pipefd[1];
 		close(pipefd[0]);
 
-		log(L_DEBUG,"Child created - I am the Child %d",pipefd[1]);
+		dbms_log(L_DEBUG,"Child created - I am the Child fd=%d",mum_fd);
 
       		FD_CLR(sockfd,&allwset);
         	FD_CLR(sockfd,&allrset);
@@ -183,17 +245,14 @@ create_new_child( void )
 		sigaction(SIGPIPE,&act,&oact);
 
                 for(r=client_list; r; r=r->next) {
+			assert(r);
 			r->type = C_LEGACY;
-			r->close=1;
+			r->close=1; MX;
 			};
 
-		log(L_DEBUG,"3 dbps");
-                close_all_dbps();
-		log(L_DEBUG,"3 cholder");
-                clean_children();
-		log(L_DEBUG,"3 done");
+                close_all_dbps(); /* XXX wrong; I should not have any ? */
+                clean_children(); 
 
-		myfree(child);
 		child = NULL;
 
 		/* make sure we listen to our mother... both for
@@ -201,33 +260,63 @@ create_new_child( void )
 		 * used (later) to pass off any connections, (re)do
 		 * an init on; etc, etc.
 		 */
-
-
 		/* XXX no error trapping */
-		handle_new_connection(mum_fd,C_MUM);
+		mum = handle_new_local_connection(mum_fd,C_MUM);
 		} 
 	else {
 		/* for the mother.. 
 		 */
+		int childfd = pipefd[0];
 		close(pipefd[1]);
 
-		log(L_DEBUG,"Child created - I am the Mother %d",pipefd[0]);
+		dbms_log(L_DEBUG,"Child created - I am the Mother fd=%d",childfd);
 
-		child->pid=pid;
-		child->r=NULL;
-		child->close=0;
-		child->num_dbs=0;
+#ifdef STATIC_BUFF
+		/* If we still have free-ed children on the list
+		 * then use those.
+	         */
+		if (free_child_list) {
+			child = free_child_list;
+			free_child_list = free_child_list->nxt;
+			free_child_list_len --;
+		} else {
+			/* Increase the keep treshold if we have to malloc often. */
+			if (free_child_list_keep < free_child_list_max)
+				free_child_list_keep += 2;
+#else
+{
+#endif
+			child = (struct child_rec *) mymalloc(sizeof(struct child_rec));
+		}
+		if (child == NULL )
+				return NULL;
+
+		/* Tie into the list of active children. */
 		child ->nxt = children;
 		children = child;
 
-		/* XXX no error trapping */
-		child->r = handle_new_connection(pipefd[0],C_CHILD);
+		/* For statistics and logging - no real reasons */
+		child_counter ++;
+		child->pid=pid;
+
+		/* Initalize the child 'real' structure */
+		child->r=NULL;
+		child->close=0;
+		child->num_dbs=0;
+
+		/* And take over the connection. 
+		 */
+		if ((child->r = handle_new_local_connection(childfd,C_CHILD)) == NULL) {
+			free_child(child);
+			return NULL;
+			};
 		}
 
 	/* return myself.. or null to signal that
-	 * we are the child
+	 * we are the child.
 	 */
         errno=0;
+	
 	return child;
 }
 
@@ -246,15 +335,9 @@ handoff_fd(
 	} cmsgbuf;
   struct cmsghdr * cmptr;
 
-  if (getpid() != mum_pid) {
-	log(L_FATAL,"Conceptual error, I should be a mother");
-	cleandown(-1);
-	exit(1);
-	};
-
-  log(L_DEBUG, "Handoff, fd=%d across %x %x",
+  assert(mum_pid == 0);
+  dbms_log(L_DEBUG, "Handoff fd=%d across on connection fd=%d to child",
 	r ? r->clientfd : 0,
-	child ? child->r : 0,
 	child->r ? child->r->clientfd : 0
 	);
 
@@ -285,8 +368,13 @@ handoff_fd(
   msg.msg_control = NULL;
   msg.msg_controllen = 0;
 
-  if (atomic_send(child->r->clientfd,&msg)<0)
+  if (atomic_send(child->r->clientfd,&msg,
+	iov[0].iov_len + iov[1].iov_len + iov[2].iov_len    )<0) {
+  	dbms_log(L_DEBUG, "Handoff fd=%d on fd=%d Fail: %s",
+		r->clientfd,child->r->clientfd,
+		strerror(errno));
 	return -1;
+	};
 
   bzero( (void *) &msg, sizeof msg);
   bzero( (void *) &cmsgbuf, sizeof cmsgbuf);
@@ -315,16 +403,24 @@ handoff_fd(
 
   *(int *)CMSG_DATA(cmptr) = r->clientfd;
 
-  if (atomic_send(child->r->clientfd,&msg)<0)
+  if (atomic_send(child->r->clientfd, &msg, 
+	iov[0].iov_len  )<0) {
+  	dbms_log(L_DEBUG, "Handoff fd=%d on fd=%d Fail: %s",
+		r->clientfd,child->r->clientfd,
+		strerror(errno));
 	return -1;
+	};
 
   /* we did it, forget about any work _we_ where doing
    * on this connection  and/or database association 
    * except perhaps for the pid..
+   * we only mark; to avoid double close if it gets
+   * re-used somehow.
    */
-  close(r->clientfd);
-  r->close = 1;
+  dbms_log(L_DEBUG, "Marking fd=%d ass closed",r->clientfd);
+  r->close = 1; MX; 
   r->type = C_LEGACY;
+  zap(r);
   return  0;
 }
   
@@ -341,13 +437,8 @@ takeon_fd(int conn_fd)
 	} cmsgbuf;
   struct cmsghdr * cmptr;
 
-  if (getpid() == mum_pid) {
-	log(L_FATAL,"Conceptual error, I should be a child");
-	cleandown(-1);
-	exit(1);
-	};
+  assert(mum_pid != 0);
 	
-  log(L_DEBUG,"Expecting a handoff.");
   /* XXX really needed ? 
    */
   bzero( (void *) &msg, sizeof msg);
@@ -372,7 +463,9 @@ takeon_fd(int conn_fd)
   /* XXX message could be '0' in size !? */
   while(1) {
 	int e=recvmsg(conn_fd,&msg,0);
-	if ((e<0) && ((errno == EAGAIN) || (errno==EINTR)))
+	if ((e<0) && (errno == EAGAIN))
+		continue;
+	if ((e<=0) && (errno==EINTR))
 		continue;
 	if (e<0)
 		return -1;
@@ -380,27 +473,32 @@ takeon_fd(int conn_fd)
 	};
 		
   if ((cmptr=CMSG_FIRSTHDR(&msg)) == NULL ) {
-	log(L_ERROR,"Not the right msg struct");
-	return -1;
-	};
-
-  if ((cmptr->cmsg_type != SCM_RIGHTS) || ( cmptr->cmsg_level != SOL_SOCKET)) {
-	log(L_ERROR,"Not the right RIGHTS/SOCKED packed");
+	dbms_log(L_ERROR,"Not the right msg struct");
 	return -1;
 	};
 
    if (cmptr->cmsg_len != CMSG_LEN(sizeof(int))) {
-	log(L_ERROR,"Not the right length of fd struct %d",
+	dbms_log(L_ERROR,"Not the right length of fd struct %d",
 		cmptr->cmsg_len);
 	return -1;
 	};
 
+  if (cmptr->cmsg_type != SCM_RIGHTS)  {
+	dbms_log(L_ERROR,"Not the right SCM_RIGHTS passed");
+	return -1;
+	};
+
+  if ( cmptr->cmsg_level != SOL_SOCKET) {
+	dbms_log(L_ERROR,"Not the right SOL_SOCKET passed");
+	return -1;
+	};
+
   fd = *(int *)CMSG_DATA(cmptr);
 
   if (fd<0)
-	log(L_FATAL,"Negative value ? %d",fd);
+	dbms_log(L_FATAL,"Negative value ? %d",fd);
 
-  log(L_VERBOSE,"Received FD=%d",fd);
+  dbms_log(L_VERBOSE,"Received FD=%d",fd);
 
   /* this is going to be follwed by an INIT type of msg so we
    * kinda are not going to handle right here. (We could do it,
