@@ -1,38 +1,22 @@
-/* DBMS Server
- * $Id: deamon.c,v 1.6 1999/01/04 22:31:56 dirkx Exp $
+/*
+ *     Copyright (c) 2000-2004 Alberto Reggiori <areggiori@webweaving.org>
+ *                        Dirk-Willem van Gulik <dirkx@webweaving.org>
  *
- * (c) 1998 Joint Research Center Ispra, Italy
- *     ISIS / STA
- *     Dirk.vanGulik@jrc.it
+ * NOTICE
  *
- * based on UKDCils
+ * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
+ * file you should have received together with this source code. If you did not get a
+ * a copy of such a license agreement you can pick up one at:
  *
- * (c) 1995 Web-Weaving m/v Enschede, The Netherlands
- *     dirkx@webweaving.org
- */
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <unistd.h>
-
-#include <fcntl.h>
-#include <time.h>
-#include <string.h>
-#include <signal.h>
-
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/errno.h>
-#include <sys/uio.h>
-
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <db.h>
+ *     http://rdfstore.sourceforge.net/LICENSE
+ *
+ *
+ * $Id: deamon.c,v 1.25 2004/08/19 18:57:36 areggiori Exp $
+ */ 
 
 #include "dbms.h"
+#include "dbms_compat.h"
+#include "dbms_comms.h"
 #include "dbmsd.h"
 
 #include "deamon.h"
@@ -40,41 +24,65 @@
 #include "mymalloc.h"
 
 static int going_down = 0;
+int client_counter = 0; /* XX static perhaps */
+
+#ifdef STATIC_BUFF
+static connection * free_connection_list = NULL;
+static int free_connection_keep = 2;
+static int free_connection_keep_max = 4;
+static int free_connection_len = 0;
+#endif
+
+#define PTOK { int i; for(i=0; i<sizeof(cmd_table) / sizeof(struct command_req); i++) if ( cmd_table[i].cmd == r->cmd.token ) { dbms_log(L_DEBUG,"Token at %s:%d %s s=%d,%d",__FILE__,__LINE__, cmd_table[i].info,r->cmd.len1,r->cmd.len2); break; }; if(i>=sizeof(cmd_table) / sizeof(struct command_req)) dbms_log(L_DEBUG,"Token at %s:%d %s %d,%d",__FILE__,__LINE__, "**UNKOWN**",r->cmd.len1,r->cmd.len2); }
 
 void 
-close_connection ( connection * r ) {
-	log(L_VERBOSE,"Closing fd %d",r->clientfd);
-
+close_connection ( connection * r ) 
+{
+	/* assert(r->clientfd); */
         FD_CLR(r->clientfd,&allwset);
         FD_CLR(r->clientfd,&allrset);
         FD_CLR(r->clientfd,&alleset);
-
 	close(r->clientfd);
+	r->clientfd = 0;
 
         /* shutdown(r->clientfd,2); */
 
-        if (r->sendbuff != NULL)
+        if (r->sendbuff != NULL) {
                 myfree(r->sendbuff);
+		r->sendbuff = NULL;
+	};
 
-        if (r->recbuff != NULL)
+        if (r->recbuff != NULL) {
                 myfree(r->recbuff);
-
-	r->recbuff = r->sendbuff = NULL;
+	 	r->recbuff = NULL;
+	};
 
 	r->send = r->tosend = 0;
 	r->gotten = r->toget = 0;
 
-	r->close = 2;
+	r->close = 2; MX;
+
+#ifdef STATIC_BUFF
+	if (free_connection_len < free_connection_keep) {
+		r->next = free_connection_list;
+		/* assert( free_connection_list != r ); */
+		free_connection_list = r;
+		free_connection_len++;
+	} else 
+#endif
+	myfree(r);
+
+	client_counter --;
+
         return;
-        }
+}
                        
 void free_connection (connection * r ) {
-	log(L_VERBOSE,"Freeing connection fd %d type=%d",r->clientfd,r->type);
 
 	if (r->type == C_MUM) {
 		/* connection to the mother lost.. we _must_ exit now.. */
 		if (!going_down) 
-			log(L_FATAL,"Mamma has died.. suicide time %d",mum_fd);
+			dbms_log(L_FATAL,"Mamma has died.. suicide time for child, fd=%d",r->clientfd);
 		cleandown(0);
 		} else
 	if (r->type == C_CLIENT) {
@@ -85,14 +93,14 @@ void free_connection (connection * r ) {
 			 */
 			r->dbp->num_cls --;
 			if (r->dbp->num_cls<=0) {
-				log(L_INFORM,"Child was the last one to use %s, closing\n", 
+				dbms_log(L_INFORM,"Child was the last one to use %s, closing", 
 					r->dbp->pfile);
-				r->dbp->close = 1;
+				r->dbp->close = 1; MX;
 				};
-			log(L_DEBUG,"Connection to client closed");
+			dbms_log(L_DEBUG,"Connection to client closed");
 			} 
 		else {
-			log(L_WARN,"C_Client marked with no database ?");
+			dbms_log(L_WARN,"C_Client marked with no database ?");
 			}
 		} else 
 	if (r->type == C_NEW_CLIENT) {
@@ -100,10 +108,10 @@ void free_connection (connection * r ) {
 		 * to send a message...
 		 * 
 		 */
-		log(L_ERROR,"New child closing.. but then what ?");
+		dbms_log(L_ERROR,"New child closing.. but then what ?");
 		} else 
 	if (r->type == C_LEGACY) {
-		log(L_DEBUG,"Legacy close");
+		dbms_log(L_DEBUG,"Legacy close");
 		} else 
 	if (r->type == C_CHILD) {
 #ifdef FORKING
@@ -116,48 +124,46 @@ void free_connection (connection * r ) {
 		for(p=first_dbp;p;p=p->nxt)
 			if (p->handled_by->r == r) {
 				if ((c) && (p->handled_by != c))
-					log(L_ERROR,"More than one child pointer ?");
-				p->close = 1;
+					dbms_log(L_ERROR,"More than one child pointer ?");
+				p->close = 1; MX;
 				c = p->handled_by;
 				};
 		if (c==NULL) {
-			log(L_ERROR,"Child died, but no record..");
+			dbms_log(L_ERROR,"Child died, but no record..");
 			} 
 		else {
 		/* overkill, as we do not even wait for the
 		 * child to be clean up after itself.
 		 */
-		c->close = 1;
+		c->close = 1; MX;
 		if (kill(c->pid,0) == 0) {
 			/* so the child is still alive ? */
-			log(L_DEBUG,"Sending kill signal (%d) to %d",
+			dbms_log(L_DEBUG,"Sending kill signal (%d) to %d",
 				SIGTERM,c->pid);
 			kill(c->pid,SIGTERM);
 			};
 		};
 #else
-		log(L_ERROR,"We are non forking, but still see a C_CHILD");
+		dbms_log(L_ERROR,"We are non forking, but still see a C_CHILD");
 #endif
 		} 
 	else {
-		log(L_ERROR,"Zapping a rather unkown connection type?");
+		dbms_log(L_ERROR,"Zapping a rather unkown connection type?");
 		};
 
 	r->type = C_UNK;
 	close_connection(r);
-	myfree(r);
 	return;
-	}
+}
 
 void zap ( connection * r ) {
 	connection * * p;
-	log(L_VERBOSE,"zapping connection %d T=%d",r->clientfd,r->type);
 
 	for ( p = &client_list ; *p && *p != r; )
 		p = &((*p)->next);
 
 	if ( *p == NULL) {
-		log(L_ERROR,"Connection to zap not found");
+		dbms_log(L_ERROR,"Connection to zap not found");
 		return;
 		};
 
@@ -166,11 +172,10 @@ void zap ( connection * r ) {
 	}
 
 
-void cleandown( int signo ) { 
-        connection * r;
-
+void cleandown( int signo ) 
+{ 
 	if (going_down) 
-		log(L_ERROR,"Re-entry of cleandown().");
+		dbms_log(L_ERROR,"Re-entry of cleandown().");
 
 	going_down = 1;
 
@@ -179,62 +184,70 @@ void cleandown( int signo ) {
 
 	/* send a kill to my children 
 	 */
-	if (getpid()==mum_pid)
+	if (!mum_pid)
 		kill(SIGTERM,0);
 
 	close_all_dbps();
 #ifdef FORKING
 	clean_children();
 #endif
-
-        for( r=client_list; r;) {
+{
+        connection * r;
+        for(r=client_list; r;) {
                 connection * q;
                 q = r; r=r->next;
+		assert(q != r);
                 close_connection(q);
-                };
-#ifdef DEBUG_MALLOC
-	debug_malloc_dump();
+        }
+}
+
+	/* close connection to mother */
+	if (mum)
+		close_connection(mum);
+
+
+#ifdef RDFSTORE_DBMS_DEBUG_MALLOC
+	debug_malloc_dump(stderr);
 #endif
-#ifdef TIME_DEBUG
+#ifdef RDFSTORE_DBMS_DEBUG_TIME
 	fprintf(stderr,"Timedebug: Time waiting=%f, handling=%f network=%f\n",.1,.2,.3);
 #endif
-	/* and call it a day. 
-	 */
-	unlink(pid_file);	
-	log(L_WARN,"Shutdown completed");
+	if (!mum_pid)
+		unlink(pid_file);	
+
+	dbms_log(L_WARN,"Shutdown completed");
 	exit(0);
 	}
 
 void continue_send( connection * r ) {
 	int s;
-	log(L_VERBOSE,"Continued send");
 
 	if ((r->tosend==0) || ( r->send >= r->tosend)) {
-		log(L_ERROR,"How did we get here ?");
-		r->close=1;
+		dbms_log(L_ERROR,"How did we get here ?");
+		r->close=1; MX;
 		return;
 		};
 
 	s = write(r->clientfd,r->sendbuff+r->send,r->tosend - r->send);
 
-	if ((s<0) && (errno == EINTR))  {
-		log(L_INFORM,"Continued send interrupted. Retry.");
+	if ((s<=0) && (errno == EINTR))  {
+		dbms_log(L_INFORM,"Continued send interrupted. Retry.");
 		return;
 		} 
 	else
 	if ((s<0) && (errno == EAGAIN))  {
-		log(L_WARN,"Continued send would still block");
+		dbms_log(L_WARN,"Continued send would still block");
 		return;
 		} 
 	else
 	if (s<0) {
-		log(L_ERROR,"Failed to continue write %s",strerror(errno));
+		dbms_log(L_ERROR,"Failed to continue write %s",strerror(errno));
 		r->close=1;
 		return;
 		}
 	else
 	if (s==0) {
-		log(L_ERROR,"Client closed the connection on us");
+		dbms_log(L_ERROR,"Client closed the connection on us");
 		r->close=1;
 		return;
 		}	
@@ -244,20 +257,22 @@ void continue_send( connection * r ) {
 		return;
 
 	r->send = r->tosend = 0;
-	myfree( r->sendbuff );
+
+#ifndef STATIC_SC_BUFF
+	if (r->sendbuff)
+		myfree( r->sendbuff );
 	r->sendbuff = NULL;
+#endif
 
 	FD_CLR(r->clientfd,&allwset);
-	log(L_DEBUG,"Done(cont) %d\n",time(NULL));
 	return;
-	}
+}
 	
 void dispatch( connection * r, int token, DBT * v1, DBT * v2) {
 	int s;
 
-	log(L_DEBUG,"Dispatching reply on fd=%d at %d",r->clientfd,time);
 	if ((r->tosend != 0) && (r->send !=0)) {
-		log(L_WARN,"dispatch, but still older data left to send");
+		dbms_log(L_WARN,"dispatch, but still older data left to send");
 		goto fail_dispatch;
 		};
 
@@ -281,7 +296,7 @@ void dispatch( connection * r, int token, DBT * v1, DBT * v2) {
 	r->cmd.len1 = htonl( r->cmd.len1 );
 	r->cmd.len2 = htonl( r->cmd.len2 );
 
-#ifdef TIME_DEBUG
+#ifdef RDFSTORE_DBMS_DEBUG_TIME
 	gettimeofday(&(r->cmd.stamp),NULL);
 #endif
 	/* BUG: we also use this with certain errors, in an attempt to
@@ -292,45 +307,63 @@ void dispatch( connection * r, int token, DBT * v1, DBT * v2) {
 
 	if (s<0) {
 		if (errno == EINTR) {
-			log(L_INFORM,"Initial write interrupted. Ignored");
+			dbms_log(L_INFORM,"Initial write interrupted. Ignored");
 			s=0;	
 			}
 		else
 		if (errno == EAGAIN) {
-			log(L_INFORM,"Initial write would block");
+			dbms_log(L_INFORM,"Initial write would block");
 			s = 0;
 			}
 		else {
-			log(L_ERROR,"Initial write error:",strerror(errno));
+			dbms_log(L_ERROR,"Initial write error: %s",strerror(errno));
 			goto fail_dispatch;
 			};
 		}
 	else
-	if (s==0) {
-		log(L_ERROR,"Intial write; client closed connection");
+	if ((s==0) && (errno != EINTR)) {
+		dbms_log(L_ERROR,"Intial write; client closed connection");
 		goto fail_dispatch;
-		};
+	};
 
 	r->send += s;
 	if (r->send == r->tosend) {
 		r->send = 0;
 		r->tosend =0;
+#ifndef STATIC_SC_BUFF
+		if (r->sendbuff)
+			myfree(r->sendbuff);
 		r->sendbuff = NULL;
-		log(L_DEBUG,"Done at %d\n",time(NULL));
+#endif
 		}
 	else {
 		int at,i; void * p;
 		/* create a buffer for the remaining data 
 		 */
 	
+#if STATIC_SC_BUFF
+		if (r->tosend-r->send > MAX_SC_PAYLOAD) {
+			dbms_log(L_ERROR,	
+				"Secondary write buffer of %d>%d bytes to big",
+				r->tosend - r->send,
+				MAX_SC_PAYLOAD
+				);
+			goto fail_dispatch;
+			};
+#else
+		assert(r->tosend > r->send );
 		r->sendbuff = mymalloc( r->tosend - r->send );
+#endif
+		assert(r->sendbuff);
+
 		if (r->sendbuff == NULL) {
-			log(L_ERROR,	
+			dbms_log(L_ERROR,	
 				"Out of memory whilst creating a secondary write buffer of %d bytes",
 				r->tosend - r->send
 				);
 			goto fail_dispatch;
-			};
+		};
+
 		for(p=r->sendbuff,i=0,at=0; i < 3; i++) {
 			if ( at > r->send ) {
 				memcpy(p, r->iov[i].iov_base,r->iov[i].iov_len);
@@ -358,14 +391,13 @@ void dispatch( connection * r, int token, DBT * v1, DBT * v2) {
 		r->send = 0;
 
 		FD_SET(r->clientfd,&allwset);
-		log(L_VERBOSE,"Secondary send");		
 		};
 
 	return;
 
 fail_dispatch:
-	log(L_WARN,"dispatch failed");
-	r->close=1;
+	dbms_log(L_WARN,"dispatch failed");
+	r->close=1;MX;
 	return;
 	}
 
@@ -380,114 +412,175 @@ void do_msg ( connection * r, int token, char * msg) {
 	}
 
 connection *
-handle_new_connection( 
+handle_new_local_connection( 
 	int clientfd, int type
 	)
 {
-	connection * new, *r;
-	int i,v;
+	struct sockaddr_in none;
+	none.sin_addr.s_addr = INADDR_NONE;
+	return handle_new_connection(clientfd, type, none);
+}
 
-	for( i=0, r=client_list ; r != NULL;  r=r->next) 
-		i++;
-	
-	if (i>HARD_MAX_CLIENTS) {
-		log(L_ERROR,"Max number of clients reached (hard max), completely ignoring");
+connection *
+handle_new_connection( 
+	int clientfd, int type, struct sockaddr_in addr
+	)
+{
+	connection * new;
+	int v;
+
+	if (client_counter > HARD_MAX_CLIENTS) {
+		dbms_log(L_ERROR,"Max number of clients reached (hard max), completely ignoring");
 		close(clientfd);
 		return NULL;
 		};
 
-	if ((new = (connection *) mymalloc(sizeof(connection))) == NULL ) {
-		log(L_ERROR,"Could not claim enough memory");
+	if (client_counter >= max_clients) {
+		connection tmp; 
+		tmp.clientfd = clientfd; 
+		tmp.close = tmp.send = tmp.tosend = tmp.gotten = tmp.toget = 0;
+		reply_log(&tmp,L_ERROR,"Too many connections fd=%d",clientfd);
 		close(clientfd);
 		return NULL;
 		};
 
-	bzero(new,sizeof(new));
-
 	if ( (v=fcntl( clientfd, F_GETFL, 0)<0) || (fcntl(clientfd, F_SETFL,v | O_NONBLOCK)<0) ) {
-		log(L_ERROR,"Coulg not make socket non blocking: %s",strerror(errno));
+		dbms_log(L_ERROR,"Could not make socket non blocking: %s",strerror(errno));
 		close(clientfd);
 		return NULL;	
 		};
 
 	FD_SET(clientfd,&allrset);
 	FD_SET(clientfd,&alleset);
+
+	/* XXX we could try to fill holes in the bit array at this point;
+	 *     and get max fd as low as possible. But it seems that the OS
+	 *     already keeps the FDs as low as it can (except for OpenBSD ??)
+	 */
 	if ( clientfd > maxfd ) 
 		maxfd=clientfd;
 
+	/* if still space, use, otherwise tack another
+	 * one to the end..
+	 */
+#if STATIC_BUFF
+	if (free_connection_list != NULL) { 
+		new = free_connection_list;
+		free_connection_list = new->next;
+		free_connection_len --;
+	} else {
+		assert(free_connection_len == 0);
+		if (free_connection_keep < free_connection_keep_max/2)
+			free_connection_keep *= 2;
+		else
+		if (free_connection_keep < free_connection_keep_max)
+			free_connection_keep += 2;
+#endif
+		if ((new = (connection *) mymalloc(sizeof(connection))) == NULL ) 
+		{
+			dbms_log(L_ERROR,"Could not claim enough memory");
+			close(clientfd);
+			return NULL;
+		};
+#if STATIC_BUFF
+	}
+#endif
+	bzero(new,sizeof(connection));
+        new->next     = client_list;
+        client_list   = new;
+
+	bzero(new,sizeof(new));
+
 	/* Copy the needed information. */
 	new->clientfd = clientfd;
-	new->my_dir   = my_dir;
+
 	new->sendbuff = NULL;
+#ifdef STATIC_SC_BUFF
+	if ((type != C_CHILD))
+		new->sendbuff = (unsigned char *) mymalloc(MAX_SC_PAYLOAD);
+#endif
 	new->recbuff  = NULL;
+#ifdef STATIC_CS_BUFF
+	if ((type != C_CHILD))
+		new->recbuff  = (unsigned char *) mymalloc(MAX_CS_PAYLOAD);
+#endif
+
 	new->dbp      = NULL;
 	new->type     = type;
+
 #ifdef TIMEOUT
 	new->start    = time(NULL);
 	new->last     = time(NULL);
 #endif
+	new->address  = addr;
+	new->close      = 0;
 	new->send = new->tosend = new->gotten = new->toget = 0;
 
-        new->next     = client_list;
-        client_list   = new;
-
-	if (i >= max_clients) {
-		log(L_ERROR,"Too many connections");
-		new->close	= 1;
-		}
-	else {
-		new->close      = 0;
-		};
-
+	client_counter ++;
 	return new;
-	}
-
-void final_read( connection * r) {
+}
 
+void final_read( connection * r) 
+{
 	r->toget = r->gotten = 0;
 	parse_request(r);	
 
+#ifndef STATIC_CS_BUFF
 	if (r->recbuff) {
 		myfree(r->recbuff);
 		r->recbuff = NULL;
 		};
-
+#endif
 	return;
-	}
+}
 
 
 void initial_read( connection * r ) {
 	struct header skip_cmd;
 	int n=0;
+
 	/* we peek, untill we have the full command buffer, and
 	 * only then do we give it any attention. This safes a
 	 * few syscalls.
 	 */
+	errno = 0;
 	n=recv(r->clientfd,&(r->cmd),sizeof(r->cmd),MSG_PEEK);
 
+	if (n<0) dbms_log(L_DEBUG,"Read fd=%d n=%d errno=%d/%s",r->clientfd,n,errno,strerror(errno));
+
 	if ((n < 0) && (errno == EAGAIN)) {
+	   dbms_log(L_ERROR,"Again read %s on %d",strerror(errno),r->clientfd);
 		return;
 		} 
 	else
-	if ((n < 0) && (errno == EINTR)) {
+	if ((n <= 0) && (errno == EINTR)) {
+	   dbms_log(L_ERROR,"Interruped read %s",strerror(errno));
 		return;
 		} 
 	else
 	if (n<0) {
 		if (errno != ECONNRESET)
-			log(L_ERROR,"Read error %s",strerror(errno));
-		r->close=1;
+			dbms_log(L_ERROR,"Read error %s",strerror(errno));
+		r->close=1;MX;
 		return;
 		} 
 	else 
 	if (n==0) {
-		log(L_INFORM,"Client side close on read %s",strerror(errno));
-		r->close=1;
+		dbms_log(L_INFORM,"Client side close on read %d/%s (fd=%d)",
+			errno,strerror(errno),r->clientfd);
+		r->close=1;MX;
 		return;
 		}
 	else
-	if ( n == sizeof(r->cmd) ) {
-#ifdef TIMEE_DEBUG
+	if ( n != sizeof(r->cmd) ) {
+		/* lets log this, as we want to get an idea if this actually happens .
+		 * seems not, BSD, on high load, SCO.
+		 */
+		dbms_log(L_WARN,"Still waitingn for those 5 bytes, gotten LESS");
+		return;
+		}
+	else {
+#ifdef RDFSTORE_DBMS_DEBUG_TIME
  		float s,m;
 		struct timeval t;
 		gettimeofday(&t,NULL);
@@ -503,7 +596,7 @@ void initial_read( connection * r ) {
 #if 0
 		if (( (r->cmd.token) & ~MASK_TOKEN ) != F_CLIENT_SIDE ) {
 			reply_log(r,L_ERROR,"Not a client side token..");
-			r->close=1;
+			r->close=1; MX;
 			return;
 			};
 #endif
@@ -515,83 +608,89 @@ void initial_read( connection * r ) {
 		r->v1.size= r->cmd.len1 = ntohl( r->cmd.len1);
 		r->v2.size= r->cmd.len2 = ntohl( r->cmd.len2);
 
+		// silly endian check.
 #if 1
 		if (r->v1.size > 2*1024*1024) {
 			reply_log(r,L_ERROR,"Size one to big");
-                        r->close=1;
+                        r->close=1; MX;
                         return;
                         };
 
 		if (r->v1.size > 2*1024*1024) {
 			reply_log(r,L_ERROR,"Size two to big");
-                        r->close=1;
+                        r->close=1; MX;
                         return;
                         };
-#endif
-#if 0
-{
-		int i;
-		for(i=0; i<sizeof(cmd_table) / sizeof(struct command_req); i++) 
-                   if ( cmd_table[i].cmd == r->cmd.token ) 
-			log(L_DEBUG,"Token %s s=%d,%d",
-				cmd_table[i].info,r->cmd.len1,r->cmd.len2);
-}
 #endif
 
-		r->recbuff = r->v2.data = r->v1.data = NULL;
+#ifndef STATIC_CS_BUFF
+		if (r->recbuff)
+			myfree(r->recbuff);
+		r->recbuff = NULL;
+#endif
+		r->v2.data = r->v1.data = NULL;
 		r->toget = r->gotten = 0;
 
 		if (r->cmd.len1 + r->cmd.len2 > 0) {
+#if STATIC_CS_BUFF
+			if (r->cmd.len1 + r->cmd.len2 > MAX_CS_PAYLOAD) {
+				reply_log(r,L_ERROR,
+					"RQ string(s) to big %d>%d bytes",
+					r->cmd.len1 + r->cmd.len2,
+					MAX_CS_PAYLOAD
+					);
+				r->close=1; MX;
+				return;
+			}
+#else
 			r->recbuff = mymalloc( r->cmd.len1 + r->cmd.len2 );
+#endif
 			if (r->recbuff == NULL) {
 				reply_log(r,L_ERROR,
 					"No Memrory for RQ string(s) %d bytes",
 					r->cmd.len1 + r->cmd.len2);
-				r->close=1;
+				r->close=1; MX;
 				return;
 				};
 			r->v1.data = r->recbuff;
 			r->v2.data = r->recbuff + r->cmd.len1;
 			r->toget = r->cmd.len1 + r->cmd.len2;
-			};
+		}
 
-
-		r->iov[0].iov_base = (void *)&skip_cmd;
+		r->iov[0].iov_base = (void *) &skip_cmd;
 		r->iov[0].iov_len = sizeof( r->cmd );
 
 		r->iov[1].iov_base = r->recbuff;
 		r->iov[1].iov_len = r->toget;
 
-reread:		n = readv( r->clientfd, r->iov, 2);
-		if ((n<0) && (errno == EINTR)) {
-			log(L_INFORM,"Interrupted readv. Ignored");
+reread:		
+	   errno = 0;
+	   n = readv( r->clientfd, r->iov, 2);
+
+		if ((n<=0) && (errno == EINTR)) {
+			dbms_log(L_INFORM,"Interrupted readv. Ignored");
 			goto reread;
 			} 
 		else
 		if ((n<0) && (errno == EAGAIN)) {
-			log(L_ERROR,"Would block. Even though we should get the cmd string. Retry");
+			dbms_log(L_ERROR,"Would block. Even though we peeked at the cmd string. Retry");
 			goto reread;
 			} 
 		else
 		if (n<0) {
-			log(L_ERROR,"Error while reading remainder");
-			r->close=1;
+			dbms_log(L_ERROR,"Error while reading remainder: (1 %s",strerror(errno));
+			r->close=1; MX;
 			return;
 			}
 		else
 		if (n==0) {
-			log(L_INFORM,"Read, but client closed");
-			r->close=1;
+			dbms_log(L_INFORM,"Read, but client closed");
+			r->close=1; MX;
 			return;
-			}
-		else
-		if ( n < sizeof( r->cmd )) {
-			reply_log(r,L_WARN,"Gotten CMD and they someone stole it");
-			r->close=1;
-			return;
-			};
+		};
+
+		assert(n >= sizeof(r->cmd));
 
-		/* book keeping... */
 		n -= sizeof(r->cmd);
 		r->gotten += n;
 
@@ -600,11 +699,8 @@ reread:		n = readv( r->clientfd, r->iov, 2);
 			return;
 			};
 		}
-	else {
-		/* lets log this, as we want to get an idea if this actually happens */
-		log(L_WARN,"Still waitingn for those 5 bytes, gotten just a few");
-		return;
-		};
+	/* should not get here.. */
+	return;
 	}
 
 void
@@ -612,35 +708,39 @@ continue_read( connection * r ) {
 	/* fill up the two buffers.. */
 	int s;
 
-	log(L_VERBOSE,"continued read for %d..",r->toget);
+	dbms_log(L_VERBOSE,"continued read for %d..",r->toget);
+	errno = 0;
 	s = read( r->clientfd, r->gotten + r->v1.data, r->toget - r->gotten);
 
-	if ((s<0) && (errno == EINTR)) {
-		log(L_INFORM,"Interrupted continued read. Ignored");
+	if ((s<=0) && (errno == EINTR)) {
+		dbms_log(L_INFORM,"Interrupted continued read. Ignored");
 		return;
 		} 
 	else
 	if (((s<0) && (errno == EAGAIN)) ) {
-		log(L_ERROR,"continued read, but nothing there");
+		dbms_log(L_ERROR,"continued read, but nothing there");
 		return;	
 		}
 	else
 	if (s<0) {
-		log(L_ERROR,"Error while reading remainder");
-		r->close=1;
+		dbms_log(L_ERROR,"Error while reading remainder: (2 %s",strerror(errno));
+		r->close=1; MX;
 		return;
 		} 
 	else
 	if (s==0) {
-		log(L_ERROR,"continued read, but client closed connection");
-		r->close=1;
+		dbms_log(L_ERROR,"continued read, but client closed connection (%d/%s)",
+			errno,strerror(errno));
+		r->close=1; MX; 
 		return;
-		}; 
+	}; 
 
 	r->gotten +=s;
-	if (r->gotten >= r->toget) {
-		final_read(r);
-		return;
-		};
 
+	if (r->gotten >= r->toget) 
+		final_read(r);
+
+	return;
 	}
+
+
