@@ -1,60 +1,27 @@
-/* $Id: deamon.h,v 1.5 1999/01/04 19:59:08 dirkx Exp $
- */
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <unistd.h>
-
-#include <fcntl.h>
-#include <time.h>
-#include <string.h>
-#include <signal.h>
-
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-#include <sys/errno.h>
-#include <sys/uio.h>
-
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <db.h>                               
-
-#include "dbms.h"
-
+/*
+ *     Copyright (c) 2000-2004 Alberto Reggiori <areggiori@webweaving.org>
+ *                        Dirk-Willem van Gulik <dirkx@webweaving.org>
+ *
+ * NOTICE
+ *
+ * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
+ * file you should have received together with this source code. If you did not get a
+ * a copy of such a license agreement you can pick up one at:
+ *
+ *     http://rdfstore.sourceforge.net/LICENSE
+ *
+ *
+ * $Id: deamon.h,v 1.12 2004/08/19 18:57:36 areggiori Exp $
+ */ 
 #ifndef _H_DEAMON
 #define _H_DEAMON
 
-#if defined(BSD)
-#define _HAS_TIME_T
-#define _HAS_SENSIBLE_SPRINTF
-#endif
+#include "dbms.h"
+#include "conf.h"
+#include "dbms_comms.h"
 
-#if defined(_HAS_TIMESPEC)
-#define TIMESPEC struct timespec
-#endif
-
-#if defined(_HAS_TIMESTRUC_T)
-#define TIMESPEC timestruc_t
-#endif
-
-#if defined(_HAS_TIME_T)
-#define TIMESPEC time_t
-#endif
-
-#if defined(_HAS_SENSIBLE_SPRINTF)
-#define STRLEN(x) (x)
-#endif
-
-#if defined(_HAS_SILLY_SPRINTF)
-#define STRLEN(x) strlen(x)
-#endif
-            
-#ifdef DEBUG
-#ifdef TIME_DEBUG
+#ifdef RDFSTORE_DBMS_DEBUG
+#ifdef RDFSTORE_DBMS_DEBUG_TIME
 extern float total_time;
 
 #define MDEBUG( x ) {  \
@@ -88,12 +55,19 @@ typedef struct dbase {
 #ifdef FORKING	
 	struct child_rec * handled_by;
 #endif
-	char * my_dir;
-	char 	* name;
 	int	  sname;
-	int	mode;
+	int	  mode;
+#ifdef STATIC_BUFF
+	char	pfile[ MAX_STATIC_PFILE ];
+	char 	name[ MAX_STATIC_NAME ];
+#else
 	char	* pfile;
+	char 	* name;
+#endif
 	DB	* handle;
+#ifdef DB_VERSION_MAJOR
+        DBC *   cursor ;
+#endif
 	int	  lastfd; /* last FD from which a cursor was set */
 	int	  num_cls; /* Number of Clients served */
 	struct dbase   * nxt;
@@ -102,10 +76,9 @@ typedef struct dbase {
 
 typedef struct connection {
 
-        int             type; /* one of C_MUM, C_CHILD, ... */
+   int	type; /* one of C_MUM, C_CHILD, ... */
 
-	int	   	clientfd;
-	char	*	my_dir;
+	int      clientfd;
 
 	struct sockaddr_in address;
 
@@ -120,7 +93,8 @@ typedef struct connection {
 	struct header	cmd;
 	struct iovec 	iov[3];
 	struct msghdr	 msg;
-	
+
+	tops		op;	/* Max operation allowed */
 	int	   	send;	/* size of the outgoing block */
 	int	   	tosend;	/* bytes send sofar.. */
 
@@ -139,9 +113,10 @@ typedef struct command_req {
 	char * info;
 	int cnt;
         void (*handler)(connection * r);
-        }  command_req;
+	tops op;
+        } command_req;
 
-extern struct command_req cmd_table[14];
+extern struct command_req cmd_table[ TOKEN_MAX ];
 
 #define	L_FATAL		-2
 #define	L_ERROR 	-1
@@ -152,7 +127,7 @@ extern struct command_req cmd_table[14];
 #define	L_DEBUG 	4
 
 void reply_log(connection * r, int level, char *fmt, ...);
-void log(int level, char *fmt, ...);
+void dbms_log(int level, char *fmt, ...);
 void trace(char *fmt, ...);
 
 extern int debug,verbose,trace_on;
@@ -175,9 +150,12 @@ void zap_child( child_rec * r);
 #endif
 
 void close_connection ( connection * r );
-//void free_connection ( connection * r );
+void free_connection ( connection * r );
 void continue_send( connection * r );
 void final_read( connection * r) ;
 void initial_read( connection * r );
 void continue_read( connection * r );
+
+extern int client_counter;
+
 #endif
